@@ -1,16 +1,16 @@
-#!/usr/bin/env python2
 from __future__ import absolute_import
 from __future__ import division
 
 import argparse
 import sys
 
-from pwn import *
+from pwnlib.elf import ELF
 from pwnlib.commandline import common
 
 parser = common.parser_commands.add_parser(
     'checksec',
-    help = 'Check binary security settings'
+    help = 'Check binary security settings',
+    description = 'Check binary security settings',
 )
 parser.add_argument(
     'elf',
@@ -38,4 +38,4 @@ def main(args):
         e = ELF(f.name)
 
 if __name__ == '__main__':
-    pwnlib.commandline.common.main(__file__)
+    common.main(__file__)
