@@ -27,9 +27,7 @@ class Module(types.ModuleType):
     def __init__(self):
         self.__file__ = __file__
         self.__name__ = __name__
-        self.num_colors = 8
-        self.has_bright = self.num_colors >= 16
-        self.has_gray = self.has_bright
+        self.num_colors = termcap.get('colors', 8) if sys.platform == 'win32' else 8
         self.when = 'auto'
         self._colors = {
             'black': 0,
@@ -61,6 +59,14 @@ class Module(types.ModuleType):
     def when(self, val):
         self._when = eval_when(val)
 
+    @property
+    def has_bright(self):
+        return self.num_colors >= 16
+    
+    @property
+    def has_gray(self):
+        return self.has_bright
+
     def _fg_color(self, c):
         c = termcap.get('setaf', c) or termcap.get('setf', c)
         if not hasattr(c, 'encode'):
